<?php
namespace Zeedhi\Framework\Process;

use Zeedhi\Framework\Cache\Type\MemcachedImpl;

class ProcessManagerImpl implements ProcessManager{
    /** @var string constant STARTED The process status. */
    private const STARTED = 'started';
    /** @var string constant FINISHED The process status. */
    private const FINISHED = 'finished';
    /** @var string constant QUEUED The process status. */
    private const QUEUED = 'queued';
    /** @var string constant ABSENT The process status. */
    private const ABSENT = 'absent';
    /** @var string constant HOST The memcached host. */
    public const HOST = '127.0.0.1';
    /** @var string constant PORT The memcached port. */
    public const PORT = 11211;
    /**
     * @var Process[] $queue The queue of process organized as a map.
     *
     * e.g. ["processId" => Process]
     */
    private $queue = [];
    /* @var MemcachedImpl $cache The cache object used to manipulate process status. */
    private $cache = null;

    /**
     * Constructor...
     *
     * @param MemcachedImpl
     */
    public function __construct(MemcachedImpl $cache){
        $this->cache = $cache;
        $this->cache->getMemcached()->addServer(self::HOST, self::PORT);
    }

    /**
     * {@inheritdoc}
     */
    public function addProcess($process){
        $id = $process->getId();
        $done = $this->cache->contains($id);

        if($done){
            return false;
        }

        $done = $this->cache->save($id, self::QUEUED);
        $this->queue[$id] = $process;

        return $done;
    }

    /**
     * {@inheritdoc}
     *
     * When the process has finished this method
     * will only return 'finished' the first time.
     *
     */
    public function getProcessStatus($id){
        $done = $this->cache->contains($id);

        if(!$done){
            return self::ABSENT;
        }

        $status = $this->cache->fetch($id);

        if($status === self::FINISHED){
            $this->discard($id);
        }

        return $status;
    }

    /**
     * {@inheritdoc}
     */
    public function run($id = false){
        $done = false;

        if(!$id){
            foreach($this->queue as $id => $process){
                if(!$done = ($process->run() && $this->cache->save($id, self::FINISHED))){
                    return false;
                }
            }
        } else {
            $done = $this->cache->contains($id);

            if(!$done){
                return false;
            }

            $done = $this->cache->save($id, self::STARTED);

            if($done){
                $done = $this->queue[$id]->run() && $this->cache->save($id, self::FINISHED);
            }
        }

        return $done;
    }

    /**
     * {@inheritdoc}
     */
    public function discard($id){
        $done = $this->cache->contains($id);

        if(!$done){
            return false;
        }

        if(($done = $this->cache->delete($id)) && isset($this->queue[$id])){
            unset($this->queue[$id]);
            return $done;
        }

        return false;
    }

    /**
     * {@inheritdoc}
     *
     * @codeCoverageIgnore
     */
    public function killAll(){
        foreach($this->queue as $id => $process){
            if($this->cache->contains($id)){
                $this->cache->delete($id);
            }
        }
        die('All processes killed!');
    }

}